// Generated by CoffeeScript 1.6.1
(function() {
  var Docco, commander, configure, defaults, document, exec, ext, fs, getLanguage, highlight, highlightEnd, highlightStart, l, languages, marked, parse, path, run, spawn, version, writeHtml, _, _ref,
    __slice = [].slice;

  document = function(config) {
    return exec("mkdir -p " + config.output, function() {
      var files, nextFile;
      exec("cp -f " + config.css + " " + config.output);
      if (fs.existsSync(config["public"])) {
        exec("cp -fR " + config["public"] + " " + config.output);
      }
      files = config.sources.slice();
      nextFile = function() {
        var source;
        source = files.shift();
        return fs.readFile(source, function(error, buffer) {
          var code, sections;
          if (error) {
            throw error;
          }
          code = buffer.toString();
          sections = parse(source, code, config);
          return highlight(source, sections, config, function() {
            writeHtml(source, sections, config);
            if (files.length) {
              return nextFile();
            }
          });
        });
      };
      return nextFile();
    });
  };

  parse = function(source, code, config) {
    var codeText, docsText, hasCode, i, lang, line, lines, match, prev, save, sections, _i, _j, _len, _len1;
    lines = code.split('\n');
    sections = [];
    lang = getLanguage(source, config);
    hasCode = docsText = codeText = '';
    save = function() {
      sections.push({
        docsText: docsText,
        codeText: codeText
      });
      return hasCode = docsText = codeText = '';
    };
    if (lang.literate) {
      for (i = _i = 0, _len = lines.length; _i < _len; i = ++_i) {
        line = lines[i];
        lines[i] = /^\s*$/.test(line) ? '' : (match = /^([ ]{4}|\t)/.exec(line)) ? line.slice(match[0].length) : lang.symbol + ' ' + line;
      }
    }
    for (_j = 0, _len1 = lines.length; _j < _len1; _j++) {
      line = lines[_j];
      if ((!line && prev === 'text') || (line.match(lang.commentMatcher) && !line.match(lang.commentFilter))) {
        if (hasCode) {
          save();
        }
        docsText += (line = line.replace(lang.commentMatcher, '')) + '\n';
        if (/^(---+|===+)$/.test(line)) {
          save();
        }
        prev = 'text';
      } else {
        hasCode = true;
        codeText += line + '\n';
        prev = 'code';
      }
    }
    save();
    return sections;
  };

  highlight = function(source, sections, config, callback) {
    var code, docs, lang, output, pygments, section;
    lang = getLanguage(source, config);
    pygments = spawn('pygmentize', ['-l', lang.name, '-f', 'html', '-O', 'encoding=utf-8,tabsize=2']);
    output = '';
    code = ((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = sections.length; _i < _len; _i++) {
        section = sections[_i];
        _results.push(section.codeText);
      }
      return _results;
    })()).join(lang.codeSplitText);
    docs = ((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = sections.length; _i < _len; _i++) {
        section = sections[_i];
        _results.push(section.docsText);
      }
      return _results;
    })()).join(lang.docsSplitText);
    pygments.stdout.on('data', function(result) {
      if (result) {
        return output += result;
      }
    });
    pygments.on('exit', function() {
      var codeFragments, docsFragments, i, _i, _len;
      output = output.replace(highlightStart, '').replace(highlightEnd, '');
      codeFragments = output ? output.split(lang.codeSplitHtml) : (function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = sections.length; _i < _len; _i++) {
          section = sections[_i];
          _results.push(_.escape(section.codeText));
        }
        return _results;
      })();
      docsFragments = marked(docs).split(lang.docsSplitHtml);
      for (i = _i = 0, _len = sections.length; _i < _len; i = ++_i) {
        section = sections[i];
        section.codeHtml = highlightStart + codeFragments[i] + highlightEnd;
        section.docsHtml = docsFragments[i];
      }
      return callback();
    });
    if (pygments.stdin.writable) {
      pygments.stdin.write(code);
      return pygments.stdin.end();
    }
  };

  writeHtml = function(source, sections, config) {
    var destination, firstBlock, hasTitle, html, title;
    destination = function(file) {
      return path.join(config.output, path.basename(file, path.extname(file)) + '.html');
    };
    firstBlock = marked.lexer(sections[0].docsText)[0];
    hasTitle = (firstBlock != null ? firstBlock.type : void 0) === 'heading';
    title = hasTitle ? firstBlock.text : path.basename(source);
    html = config.template({
      sources: config.sources,
      css: path.basename(config.css),
      title: title,
      hasTitle: hasTitle,
      sections: sections,
      path: path,
      destination: destination
    });
    console.log("docco: " + source + " -> " + (destination(source)));
    return fs.writeFileSync(destination(source), html);
  };

  defaults = {
    layout: 'parallel',
    output: 'docs/',
    template: null,
    css: null,
    extension: null
  };

  configure = function(options) {
    var config, dir;
    config = _.extend({}, defaults, _.pick.apply(_, [options].concat(__slice.call(_.keys(defaults)))));
    if (options.template || options.css) {
      config.layout = null;
    } else {
      dir = config.layout = "" + __dirname + "/resources/" + config.layout;
      if (fs.existsSync("" + dir + "/public")) {
        config["public"] = "" + dir + "/public";
      }
      config.template = "" + dir + "/docco.jst";
      config.css = "" + dir + "/docco.css";
    }
    config.template = _.template(fs.readFileSync(config.template).toString());
    config.sources = options.args.filter(function(source) {
      var lang;
      lang = getLanguage(source, config);
      if (!lang) {
        console.warn("docco: skipped unknown type (" + m + ")");
      }
      return lang;
    }).sort();
    return config;
  };

  _ = require('underscore');

  fs = require('fs');

  path = require('path');

  marked = require('marked');

  commander = require('commander');

  _ref = require('child_process'), spawn = _ref.spawn, exec = _ref.exec;

  languages = JSON.parse(fs.readFileSync("" + __dirname + "/resources/languages.json"));

  for (ext in languages) {
    l = languages[ext];
    l.commentMatcher = RegExp("^\\s*" + l.symbol + "\\s?");
    l.commentFilter = /(^#![/]|^\s*#\{)/;
    l.codeSplitText = "\n" + l.symbol + "DIVIDER\n";
    l.codeSplitHtml = RegExp("\\n*<span\\sclass=\"c1?\">" + l.symbol + "DIVIDER<\\/span>\\n*");
    l.docsSplitText = "\n#" + l.name + "DOCDIVIDER\n";
    l.docsSplitHtml = RegExp("<h1>" + l.name + "DOCDIVIDER</h1>");
  }

  getLanguage = function(source, config) {
    var codeExt, codeLang, lang;
    ext = config.extension || path.extname(source);
    lang = languages[ext];
    if (lang && lang.name === 'markdown') {
      codeExt = path.extname(path.basename(source, ext));
      if (codeExt && (codeLang = languages[codeExt])) {
        lang = _.extend({}, codeLang, {
          literate: true
        });
      }
    }
    return lang;
  };

  highlightStart = '<div class="highlight"><pre>';

  highlightEnd = '</pre></div>';

  version = JSON.parse(fs.readFileSync("" + __dirname + "/package.json")).version;

  run = function(args) {
    var d;
    if (args == null) {
      args = process.argv;
    }
    d = defaults;
    commander.version(version).usage('[options] files').option('-l, --layout [name]', 'choose a built-in layout (parallel or linear)', d.parallel).option('-o, --output [path]', 'output to a given folder', d.output).option('-c, --css [file]', 'use a custom css file', d.css).option('-t, --template [file]', 'use a custom .jst template', d.template).option('-e, --extension [ext]', 'assume a file extension for all inputs', d.extension).parse(args).name = "docco";
    if (commander.args.length) {
      return document(configure(commander));
    } else {
      return console.log(commander.helpInformation());
    }
  };

  Docco = module.exports = {
    run: run,
    document: document,
    parse: parse,
    version: version
  };

}).call(this);
